# Part 2: 파이썬 핵심 문법 마스터하기

---

### 💡 지난 시간 복습

[Part 1: AI 개발 환경 구축](part_1_ai_development_environment.md)에서는 VSCode, Jupyter, Colab 등 다양한 개발 도구의 특징을 알아보고, 프로젝트의 독립성을 보장하는 **가상환경**의 중요성과 사용법을 익혔습니다. 이제 우리는 코드를 작성하고 실행할 준비를 마쳤습니다.

---

AI 개발의 세계에 오신 것을 환영합니다! 파이썬은 AI와 데이터 과학 분야에서 가장 사랑받는 언어입니다. 문법이 간결하고 사람의 생각과 비슷해서, 프로그래밍을 처음 시작하는 분들도 쉽게 배울 수 있습니다. 이 문서는 [점프 투 파이썬](https://wikidocs.net/book/1)을 참고하여 파이썬을 처음 접하시거나, 다시 복습하고 싶은 분들을 위해 가장 핵심적인 문법만을 골라 친절하게 설명합니다.

## 1. 변수와 자료형: 데이터 다루기의 첫걸음

### 1.1. 변수(Variable): 데이터에 이름표 붙이기

프로그래밍은 결국 '데이터'를 다루는 일입니다. **변수(Variable)**는 데이터에 이름을 붙이고 저장하는 '라벨이 붙은 상자'와 같습니다. 상자에 물건을 넣고 필요할 때 꺼내 쓰는 것처럼, 변수에 데이터를 저장하고 필요할 때마다 이름을 불러 사용합니다.

```python
# 'box'라는 이름의 상자(변수)에 숫자 100을 저장합니다.
box = 100
print(box) # 100

# 이제 'box'에 문자열 "Hello"를 저장합니다. 상자의 내용물이 바뀌었습니다.
box = "Hello"
print(box) # "Hello"
```

파이썬은 매우 유연해서, 상자에 어떤 종류의 물건이 들어갈지 미리 말해주지 않아도 됩니다. 숫자를 넣으면 숫자 상자가, 글자를 넣으면 글자 상자가 되는 식이죠. 이를 **동적 타이핑(Dynamic Typing)**이라고 부릅니다.


### 1.2. 자료형(Data Types): 데이터의 종류

파이썬은 다양한 종류의 데이터를 다룰 수 있습니다. 이를 **자료형**이라고 부릅니다.

#### 1.2.1. 숫자형 (Numeric)
정수(`int`), 실수(`float`)는 계산을 위해 사용되는 가장 기본적인 자료형입니다.

```python
# 정수(Integer)와 실수(Float)
a = 10
b = 3.14

# 기본적인 사칙연산
print(f"덧셈: {a + b}")
print(f"나눗셈: {a / b}")

# 프로그래밍에서만 볼 수 있는 연산
print(f"나눗셈 후 나머지: {10 % 3}")     # 1
print(f"나눗셈 후 몫만: {10 // 3}")      # 3
print(f"거듭제곱: {2 ** 4}")  # 2의 4제곱 = 16
```

> 💡 **Tip: 파이썬 3의 나눗셈**
> 파이썬 2에서는 `10 / 3`의 결과가 `3`(정수)이었지만, 파이썬 3부터는 `/` 연산자가 항상 `float`(실수)를 반환합니다 (`3.333...`). 정수 나눗셈의 몫을 원할 때는 `//` 연산자를 사용해야 합니다. 이 차이점을 기억하는 것은 매우 중요합니다.

#### 1.2.2. 문자열 (String)
'글자'들의 나열입니다. 작은따옴표(`'`) 또는 큰따옴표(`"`)로 감싸서 만듭니다.

```python
greeting = "Hello, Python!"
# f-string을 사용하면 문자열 안에 {변수명} 형태로 값을 쉽게 넣을 수 있습니다.
name = "앨리스"
print(f"안녕하세요, {name}님!")

# 인덱싱 (Indexing): 특정 위치의 글자 하나를 가져오기 (0부터 시작!)
first_char = greeting[0] # 'H'

# 슬라이싱 (Slicing): [시작:끝] 형태로 범위를 지정하여 여러 글자를 가져오기
py_string = greeting[7:13] # 'Python'

# 주요 문자열 메서드
text = "  Welcome to Python!  "
print(f"대문자로: {text.upper()}")
print(f"공백 제거: '{text.strip()}'")
print(f"치환: {text.replace('Python', 'World')}")
print(f"분리: {text.strip().split(' ')}")
```

> 💡 **Tip: f-string을 생활화하세요!**
> 과거에는 `"... %s ..." % name` 이나 `"... {}".format(name)` 같은 방식으로 문자열을 포맷팅했지만, f-string (`f"..."`)은 훨씬 간결하고, 직관적이며, 빠릅니다. 특별한 이유가 없다면 항상 f-string을 사용하는 것이 현대 파이썬 코딩 스타일입니다.

#### 1.2.3. 불리언 (Boolean)
`True`(참)와 `False`(거짓) 단 두 가지 값만 가지는 특별한 자료형입니다. 조건 판단의 기준이 됩니다.

```python
is_active = True
has_permission = False

# 논리 연산자: and, or, not
print(f"is_active and has_permission: {is_active and has_permission}") # False
print(f"10 > 5: {10 > 5}") # True
```

> 🚫 **Common Pitfall: `==` vs `is`**
> 파이썬 초보자들이 가장 흔하게 혼동하는 것 중 하나가 `==`와 `is`의 차이입니다.
> - `==` (동등 연산자): 두 변수의 **값**이 같은지 비교합니다. (예: `[1, 2]` == `[1, 2]` -> `True`)
> - `is` (식별 연산자): 두 변수가 **완전히 동일한 객체(메모리 주소)**를 가리키는지 비교합니다. (예: `[1, 2]` is `[1, 2]` -> `False`, 서로 다른 리스트 객체이므로)
>
> 일반적으로는 값의 일치 여부를 확인하는 `==`를 사용하는 것이 안전하고, `None`이나 `True`, `False`와 같이 유일한 객체임을 보장할 수 있는 경우에만 `is`를 사용하는 것이 좋습니다. (예: `if my_var is None:`)

### 1.3. 여러 데이터를 묶어서 관리하는 자료구조

하나가 아닌 여러 개의 데이터를 효율적으로 관리하기 위한 자료형입니다.

#### 1.3.1. 리스트 (List)
- 여러 개의 값을 순서대로 저장하는 **변경 가능한** 자료구조입니다.
- 대괄호 `[]`로 감싸고, 각 요소는 쉼표 `,`로 구분합니다.

```python
# 리스트 생성
fruits = ["apple", "banana", "cherry"]
numbers = [1, 2, 3, 4, 5]

# 인덱싱과 슬라이싱 (문자열과 동일)
print(f"첫 번째 과일: {fruits[0]}") # apple
print(f"마지막 두 개 과일: {fruits[1:]}") # ['banana', 'cherry']

# 리스트 값 변경하기
fruits[1] = "blueberry"
print(f"변경된 과일 리스트: {fruits}") # ['apple', 'blueberry', 'cherry']

# 리스트에 요소 추가/제거
fruits.append("strawberry") # 맨 뒤에 추가
print(f"추가 후: {fruits}")
fruits.remove("apple") # 특정 값 제거
print(f"제거 후: {fruits}")

# 리스트 길이 구하기
print(f"리스트의 길이: {len(fruits)}")

> 💡 **Tip: 리스트 컴프리헨션(List Comprehension)으로 파이썬답게 코딩하기**
> `for` 반복문과 `append`를 사용하여 리스트를 만드는 것은 직관적이지만, 파이썬에서는 더 간결하고 가독성 높은 방법이 있습니다. 바로 **리스트 컴프리헨션**입니다.
>
> ```python
> # 0부터 9까지의 숫자 중 짝수만 제곱하여 리스트 만들기
>
> # 일반적인 for문 사용
> squares = []
> for i in range(10):
>     if i % 2 == 0:
>         squares.append(i**2)
> print(squares) # [0, 4, 16, 36, 64]
>
> # 리스트 컴프리헨션 사용
> squares_comp = [i**2 for i in range(10) if i % 2 == 0]
> print(squares_comp) # [0, 4, 16, 36, 64]
> ```
> 리스트 컴프리헨션을 사용하면 한 줄로 동일한 작업을 수행할 수 있어 코드가 훨씬 깔끔해집니다. 익숙해지면 코드를 읽고 쓰는 속도가 매우 빨라집니다.

#### 1.3.2. 튜플 (Tuple)
- 여러 개의 값을 순서대로 저장하는 **변경 불가능한** 자료구조입니다.
- 소괄호 `()`로 감싸서 만듭니다. 한번 만들어지면 내용을 바꿀 수 없어서, 안정성이 중요할 때 사용됩니다.

```python
# 튜플 생성
point = (10, 20)
colors = ("red", "green", "blue")

# 인덱싱과 슬라이싱 (리스트와 동일)
print(f"x좌표: {point[0]}")

# 튜플은 변경 불가! 아래 코드는 오류를 발생시킵니다.
# point[0] = 15 # TypeError
```

#### 1.3.3. 딕셔너리 (Dictionary)
- `Key:Value` 쌍으로 데이터를 저장하는 순서가 없는 자료구조입니다.
- 중괄호 `{}`로 감싸서 만듭니다. '이름'='홍길동', '나이'=30 처럼 의미를 가진 데이터를 관리할 때 매우 유용합니다.

```python
# 딕셔너리 생성
user = {
    "name": "앨리스",
    "age": 30,
    "city": "New York"
}

# Key를 이용해 Value에 접근
print(f"이름: {user['name']}")
print(f"나이: {user.get('age')}")

# 데이터 추가 및 변경
user['email'] = "alice@example.com" # 새로운 Key-Value 쌍 추가
user['age'] = 31 # 기존 Key의 Value 변경
print(f"수정된 사용자 정보: {user}")

# 데이터 삭제
del user['city']
print(f"삭제 후 사용자 정보: {user}")

# 모든 Key와 Value 보기
print(f"키 목록: {list(user.keys())}")
print(f"값 목록: {list(user.values())}")
```

#### 1.3.4. 집합 (Set)
- **중복을 허용하지 않는** 순서 없는 데이터의 모음입니다.
- 중괄호 `{}`를 사용하지만, Key가 없는 점이 딕셔너리와 다릅니다.
- 주로 데이터의 중복을 제거하거나, 여러 데이터 그룹 간의 연산(합집합, 교집합 등)에 사용됩니다.

```python
# 집합 생성
unique_numbers = {1, 2, 3, 2, 1, 4}
print(f"중복이 제거된 집합: {unique_numbers}") # {1, 2, 3, 4}

set_a = {1, 2, 3, 4}
set_b = {3, 4, 5, 6}

# 집합 연산
print(f"합집합 (A | B): {set_a | set_b}") # {1, 2, 3, 4, 5, 6}
print(f"교집합 (A & B): {set_a & set_b}") # {3, 4}
print(f"차집합 (A - B): {set_a - set_b}") # {1, 2}
```

---

## 2. 제어문: 코드의 흐름 조종하기

**제어문(Control Flow)**을 사용하면 특정 조건에 따라 코드를 실행하거나, 반복 실행하는 등 코드의 실행 흐름을 통제할 수 있습니다.

### 2.1. 파이썬의 핵심, 들여쓰기 (Indentation)
파이썬은 **들여쓰기(indentation)**로 코드 블록을 구분합니다. 보통 4개의 공백(스페이스바 4번)을 사용하며, 이는 파이썬 문법의 필수 요소이므로 매우 중요합니다.

### 2.2. if-elif-else (조건문)
조건에 따라 다른 코드를 실행하고 싶을 때 사용합니다.

```python
score = 85
if score >= 90:
    grade = "A"
elif score >= 80:
    grade = "B"
else:
    grade = "F"
print(f"당신의 학점은 {grade}입니다.")
```

### 2.3. for (반복문)
리스트, 튜플, 문자열 등 순회 가능한(iterable) 객체의 요소를 하나씩 순회하며 코드를 반복 실행합니다.

```python
fruits = ["apple", "banana", "cherry"]

for fruit in fruits:
    print(f"나는 {fruit}를 좋아합니다!")

# enumerate를 사용하면 인덱스 번호와 값을 함께 사용할 수 있습니다.
for idx, fruit in enumerate(fruits):
    print(f"{idx+1}번째 과일은 {fruit}입니다.")

# range() 함수는 정해진 횟수만큼 반복할 때 유용합니다.
for i in range(5): # 0부터 4까지 5번 반복
    print(f"반복 횟수: {i}")
```

### 2.4. while (반복문)
특정 조건이 `True`인 **동안** 계속해서 코드를 반복 실행합니다. `while`문은 무한 루프에 빠질 위험이 있으니 항상 주의해야 합니다.

```python
count = 5
while count > 0:
    print(f"카운트다운: {count}")
    count -= 1
print("발사!")

# break(탈출)와 continue(건너뛰기)로 반복 제어
i = 0
while True: # 의도적인 무한 루프
    i += 1
    if i % 2 == 0:
        continue # 짝수이면 아래 print를 건너뛰고 다음 반복으로
    print(i) # 홀수만 출력
    if i >= 10:
        break # 10 이상이 되면 반복 중단
```

---

## 3. 함수: 코드 재사용을 위한 레시피

**함수(Function)**는 특정 작업을 수행하는 코드 묶음에 이름을 붙여놓은 것입니다. 함수를 사용하면 코드가 간결해지고(DRY: Don't Repeat Yourself), 수정 및 관리가 쉬워집니다.

### 3.1. 함수 정의와 호출
```python
# 'greet'라는 이름의 함수를 정의합니다.
# 'name'은 함수에 필요한 재료(인자, parameter)입니다.
def greet(name: str) -> str:
    """사용자에게 인사하는 문자열을 반환합니다.""" # Docstring
    return f"안녕하세요, {name}님!"

# 함수를 호출하여 사용합니다.
message = greet("파이썬")
print(message)
```
* **타입 힌트(Type Hint)**: `name: str`이나 `-> str` 처럼 함수의 인자나 반환값의 타입을 명시하여 코드의 가독성과 안정성을 높일 수 있습니다. (강력 권장)

### 3.2. 다양한 인자(Argument) 전달 방식
```python
# 위치 인자, 키워드 인자, 기본값 인자
def introduce(name, age=30):
    print(f"이름: {name}, 나이: {age}")

introduce("홍길동")           # age는 기본값 30 사용
introduce("이순신", age=40) # 키워드 인자

# 가변 인자 (*args, **kwargs)
# *args: 여러 개의 위치 인자를 '튜플'로 받음
def sum_all(*numbers):
    return sum(numbers)

# **kwargs: 여러 개의 키워드 인자를 '딕셔너리'로 받음
def print_user_profile(**user_info):
    for key, value in user_info.items():
        print(f"- {key}: {value}")

print(f"합계: {sum_all(1, 2, 3, 4, 5)}")
print_user_profile(name="John Doe", city="New York")
```

### 3.3. 함수의 반환값은 여러 개일 수 있으며, 이 경우 튜플로 반환됩니다.
```python
def greet_user(name, email):
    return f"{name}님, 환영합니다!", f"이메일: {email}"

# 함수의 반환값은 여러 개일 수 있으며, 이 경우 튜플로 반환됩니다.
welcome_message, email_info = greet_user("홍길동", "hong@example.com")
print(welcome_message)
print(email_info)

🚫 **Common Pitfall: 함수 기본값으로 mutable 객체 사용하기**
파이썬에서 가장 까다로운 함정 중 하나입니다. 함수의 기본값 인자로 리스트(`[]`)나 딕셔너리(`{}`) 같은 **변경 가능한(mutable)** 객체를 사용하면 예상치 못한 결과를 초래할 수 있습니다.

```python
# 잘못된 예시
def add_to_list(item, my_list=[]):
    my_list.append(item)
    return my_list

# 함수 호출
print(add_to_list(1)) # 결과: [1] (예상대로)
print(add_to_list(2)) # 결과: [1, 2] (예상과 다름! [2]가 아님)
print(add_to_list(3)) # 결과: [1, 2, 3]

# 올바른 패턴
def add_to_list_safe(item, my_list=None):
    if my_list is None:
        my_list = []
    my_list.append(item)
    return my_list

print(add_to_list_safe(1)) # [1]
print(add_to_list_safe(2)) # [2]
```
- **왜 이런 일이?**: 함수 기본값은 함수가 **정의될 때 단 한 번만** 생성됩니다. `my_list=[]`는 함수가 처음 메모리에 올라갈 때 빈 리스트를 만들고, 이후 모든 호출에서 그 *동일한 리스트*를 계속 공유하여 사용하기 때문에 위와 같은 문제가 발생합니다.
- **해결책**: 기본값으로 `None`을 사용하고, 함수 내부에서 `if my_list is None:` 과 같이 체크하여 필요할 때 새로운 객체를 생성하는 것이 안전하고 표준적인 방법입니다.

---

### 🚀 다음 시간엔...

지금까지 파이썬의 가장 기본적인 구성 요소인 변수, 자료형, 제어문, 함수에 대해 알아보았습니다. 이것들은 우리가 앞으로 만들 모든 프로그램의 뼈대가 될 것입니다.

다음 [Part 3: 데이터 관리를 위한 파이썬 컬렉션](part_3_python_collections.md)에서는 여러 개의 데이터를 한 번에 담고 효율적으로 관리하는 리스트(List), 튜플(Tuple), 딕셔너리(Dictionary), 셋(Set) 자료구조에 대해 깊이 있게 탐구해 보겠습니다.