# Part 5: AI 개발 핵심 라이브러리

---

### 💡 지난 시간 복습

[Part 4: 객체 지향 프로그래밍 (OOP)의 이해](part_4_object_oriented_programming.md)에서는 클래스, 객체, 상속, 다형성 등 OOP의 핵심 개념을 배웠습니다. 이를 통해 AI 라이브러리들이 어떻게 구조적으로 설계되었는지 이해하고, 재사용성이 높은 코드를 작성할 수 있는 기반을 마련했습니다.

---

## 0. 핵심 라이브러리 관계도 (The Big Picture)

본격적으로 각 라이브러리를 배우기 전에, 이들이 데이터 분석 및 머신러닝 프로젝트에서 어떻게 함께 사용되는지에 대한 전체적인 그림을 이해하는 것이 중요합니다. 아래 다이어그램은 일반적인 데이터 흐름을 보여줍니다.

```mermaid
graph TD
    subgraph "데이터 수집 및 전처리"
        A["<b>Pandas (DataFrame)</b><br/>- CSV/Excel/DB에서 데이터 로드<br/>- 결측치 처리, 필터링, 그룹화"]
    end

    subgraph "고성능 수치 연산"
        B["<b>NumPy (ndarray)</b><br/>- DataFrame에서 변환<br/>- 벡터/행렬 연산, 수학 함수"]
    end

    subgraph "머신러닝 모델링"
        C["<b>Scikit-learn (Estimator)</b><br/>- 모델 학습(fit), 예측(predict)<br/>- 모델 평가, 하이퍼파라미터 튜닝"]
    end

    subgraph "데이터 시각화"
        D["<b>Matplotlib & Seaborn</b><br/>(Figure, Axes)<br/>- 데이터 탐색(EDA)<br/>- 모델 결과 시각화"]
    end

    A -->|"데이터 정제"| A
    A -->|"df.values, df['column']"| B
    B -->|"모델 입력"| C
    A -->|"df.plot()"| D
    B -->|"plt.plot(array)"| D
    C -->|"결과 분석"| D
```
- **시작은 Pandas**: 대부분의 프로젝트는 Pandas를 사용해 다양한 소스(CSV, Excel 등)로부터 데이터를 불러와 `DataFrame`으로 만드는 것에서 시작합니다.
- **계산을 위해 NumPy로**: 정제된 데이터는 고성능 수치 계산을 위해 NumPy 배열(`ndarray`)로 변환되어 머신러닝 모델의 입력으로 준비됩니다.
- **Scikit-learn으로 모델링**: NumPy 배열로 변환된 데이터를 사용하여 Scikit-learn으로 모델을 학습하고 예측합니다.
- **모든 과정에서 시각화**: Pandas와 NumPy 데이터는 Matplotlib/Seaborn을 통해 시각화되어 데이터의 특징을 파악(EDA)하거나 모델의 성능을 분석하는 데 사용됩니다.

이 흐름을 염두에 두고 각 라이브러리를 학습하면, 지식이 파편적으로 남지 않고 하나의 큰 그림으로 연결될 것입니다.

---

## 1. NumPy (Numerical Python) - 고성능 수치 계산

- **핵심**: 파이썬에서 과학 계산을 위한 가장 기본적인 패키지입니다. `ndarray`(N-dimensional array)라는 효율적인 다차원 배열 객체를 중심으로, 벡터, 행렬 등 고성능 수치 연산을 위한 방대한 함수들을 제공합니다. Pandas, Scikit-learn, TensorFlow 등 대부분의 데이터 과학 라이브러리가 NumPy를 기반으로 동작하므로, AI 개발자에게는 알파벳과도 같은 라이브러리입니다.

### 1.1. NumPy 배열 생성
```python
import numpy as np

# 파이썬 리스트로부터 배열 생성
list_a = [1, 2, 3, 4, 5]
np_a = np.array(list_a)
print(f"파이썬 리스트: {list_a}")
print(f"NumPy 배열: {np_a}")

# 2차원 배열 (행렬) 생성
list_2d = [[1, 2, 3], [4, 5, 6]]
matrix = np.array(list_2d)
print("2x3 행렬:\n", matrix)

# 배열의 형태(shape), 차원(ndim), 데이터 타입(dtype) 확인
print(f"Shape: {matrix.shape}, Dtype: {matrix.dtype}, Ndim: {matrix.ndim}")

# 특정 값으로 채워진 배열 생성
zeros = np.zeros((2, 4)) # 2x4 크기의 0으로 채워진 배열
ones = np.ones((3, 2))  # 3x2 크기의 1로 채워진 배열
print("Zeros:\n", zeros)
print("Ones:\n", ones)
```

### 1.2. 배열 연산 (Element-wise Operations)
NumPy 배열의 가장 큰 장점 중 하나는 반복문 없이 배열 전체에 대해 빠른 연산이 가능하다는 것입니다.

```python
arr1 = np.array([[1, 2], [3, 4]])
arr2 = np.array([[5, 6], [7, 8]])

# 배열 간 덧셈
print("덧셈:\n", arr1 + arr2)
# print("덧셈:\n", np.add(arr1, arr2)) # 동일한 연산

# 배열 간 곱셈
print("곱셈:\n", arr1 * arr2)

# 스칼라(상수) 연산
print("스칼라 곱셈:\n", arr1 * 10)
```

### 1.3. 브로드캐스팅 (Broadcasting)
서로 다른 크기의 배열 간에도 연산이 가능하도록 배열을 자동으로 확장시켜주는 기능입니다.
```python
matrix = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
vector = np.array([10, 20, 30])

# 3x3 행렬과 1x3 벡터의 덧셈
# vector가 3x3 행렬처럼 확장되어 각 행에 더해짐
result = matrix + vector
print("브로드캐스팅 결과:\n", result)
```

### 1.4. 인덱싱과 슬라이싱
```python
arr = np.arange(10) # 0부터 9까지의 숫자로 배열 생성
print("원본 배열:", arr)

# 슬라이싱
print("arr[2:5]:", arr[2:5]) # [2 3 4]

# 2차원 배열 인덱싱
matrix = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
print("matrix[1, 2]:", matrix[1, 2]) # 2행 3열의 원소 -> 6

# 조건(Boolean) 인덱싱: 특정 조건을 만족하는 원소만 추출
evens = arr[arr % 2 == 0]
print("배열의 짝수:", evens)
```
---
## 2. Pandas - 체계적인 데이터 분석 및 정제

- **핵심**: `DataFrame`이라는 2차원 테이블 형식의 자료구조와 `Series`라는 1차원 배열 형태의 자료구조를 사용하여, 엑셀이나 데이터베이스 테이블처럼 데이터를 직관적으로 다룰 수 있게 해줍니다. CSV/Excel 파일 로딩, 데이터 필터링, 결측치 처리, 그룹화(Group by) 등 데이터 전처리에 없어서는 안 될 라이브러리입니다.

### 2.1. Series와 DataFrame 생성
```python
import pandas as pd

# Series 생성 (1차원)
s = pd.Series([10, 20, 30, 40], index=['a', 'b', 'c', 'd'])
print("Pandas Series:\n", s)

# DataFrame 생성 (2차원)
data = {'제품': ['사과', '바나나', '체리', '딸기'],
        '가격': [1000, 500, 2000, 1500],
        '재고': [10, 20, 15, 5]}
df = pd.DataFrame(data)
print("Pandas DataFrame:\n", df)
```

### 2.2. 데이터 선택 및 필터링
```python
# 특정 열(Column) 선택
prices = df['가격']
print("가격 열:\n", prices)

# loc: 라벨(이름) 기반 선택
print("1번 인덱스 행:\n", df.loc[1])
print("0, 2번 인덱스 행의 '제품', '재고' 열:\n", df.loc[[0, 2], ['제품', '재고']])

# iloc: 정수 인덱스 기반 선택
print("첫 번째 행:\n", df.iloc[0])

# 조건 필터링
high_price_products = df[df['가격'] >= 1000]
print("가격이 1000원 이상인 제품:\n", high_price_products)

low_stock_products = df[(df['가격'] < 1000) & (df['재고'] < 10)]
print("가격 1000원 미만이고 재고 10개 미만인 제품:\n", low_stock_products)

> 🚫 **Common Pitfall: `SettingWithCopyWarning`을 조심하세요!**
> Pandas를 사용하다 보면 `SettingWithCopyWarning`이라는 경고를 자주 마주치게 됩니다. 이는 버그는 아니지만, 내가 의도한 대로 원본 DataFrame이 수정되지 않았을 수 있다는 중요한 신호입니다.
>
> **언제 발생하나요?** 주로 '연쇄 인덱싱(Chained Indexing)'을 사용하여 값을 할당할 때 발생합니다.
> ```python
> # 이런 방식은 경고를 유발할 수 있습니다.
> df[df['가격'] >= 1000]['재고'] = 0
> ```
> - **왜 문제인가요?**: `df[df['가격'] >= 1000]`는 원본의 '뷰(View)'를 반환할 수도, '복사본(Copy)'을 반환할 수도 있습니다. 만약 복사본이 반환되면, 그 복사본의 '재고' 열을 0으로 바꾸는 것이므로 원본 `df`에는 아무런 변화가 없습니다. Pandas는 이처럼 사용자의 의도가 불분명한 상황을 경고해주는 것입니다.
> - **올바른 해결책**: `.loc`를 사용하여 행과 열을 한 번에 지정하는 것이 가장 확실하고 권장되는 방법입니다.
> ```python
> # .loc[행 인덱서, 열 인덱서] 형태로 한 번에 접근합니다.
> condition = df['가격'] >= 1000
> df.loc[condition, '재고'] = 0 # 경고 없이 명확하게 원본을 수정합니다.
> ```
> 이 경고를 무시하지 말고, `.loc`를 사용하는 습관을 들이는 것이 데이터 조작의 정확성을 보장하는 길입니다.
```

### 2.3. 데이터 그룹화 (Group by)
```python
data = {'팀': ['A', 'B', 'A', 'B', 'A'],
        '선수': ['손흥민', '김민재', '황희찬', '이강인', '조규성'],
        '득점': [12, 1, 8, 5, 10]}
df_player = pd.DataFrame(data)

# '팀' 별로 그룹화하여 득점의 합계 계산
team_scores = df_player.groupby('팀')['득점'].sum()
print("팀별 총 득점:\n", team_scores)

💡 **Tip: `pipe()`로 깔끔한 데이터 처리 파이프라인 만들기**
> 여러 단계의 데이터 전처리 작업을 순차적으로 적용할 때, 코드가 지저분해지거나 중간 과정에 불필요한 변수들이 많이 생길 수 있습니다. `pipe()` 메서드를 사용하면 함수들을 체인처럼 연결하여 가독성 높은 파이프라인을 구축할 수 있습니다.
>
> ```python
> # 전처리를 위한 함수들을 정의합니다.
> def add_vat(df: pd.DataFrame) -> pd.DataFrame:
>     """가격에 부가세를 추가하는 함수"""
>     df_copy = df.copy()
>     df_copy['가격(VAT포함)'] = df_copy['가격'] * 1.1
>     return df_copy
>
> def label_stock(df: pd.DataFrame) -> pd.DataFrame:
>     """재고 수준을 라벨링하는 함수"""
>     df_copy = df.copy()
>     df_copy['재고상태'] = pd.cut(df_copy['재고'], bins=[-1, 5, 15, 100], labels=['부족', '보통', '충분'])
>     return df_copy
>
> # pipe()를 사용하여 함수들을 순서대로 적용합니다.
> processed_df = (df.pipe(add_vat)
>                   .pipe(label_stock)
>                   .sort_values(by='가격'))
>
> print("파이프라인 처리 결과:\n", processed_df)
> ```
> 위 코드처럼 `(df.pipe(...).pipe(...))` 형태로 코드를 작성하면, 데이터가 어떤 순서로 처리되는지 명확하게 파악할 수 있어 유지보수가 쉬워집니다.
```
---
## 3. Matplotlib & Seaborn - 데이터 시각화

- **핵심**: 데이터를 그래프로 표현하여 패턴과 인사이트를 찾는 데 사용됩니다. `Matplotlib`은 파이썬 시각화의 근간이 되는 라이브러리로, 세밀한 조정이 가능한 다양한 종류의 그래프를 그릴 수 있습니다. `Seaborn`은 Matplotlib을 기반으로 더 아름답고 통계적으로 복잡한 그래프를 쉽게 그릴 수 있도록 돕는 고수준 라이브러리입니다.

```python
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd

# 한글 폰트 설정 (Windows: Malgun Gothic, Mac: AppleGothic)
# 실행 환경에 맞는 폰트 이름을 지정해야 합니다. 
# 폰트가 없을 경우, 오류 대신 경고 메시지를 출력하고 기본 폰트를 사용합니다.
try:
    plt.rcParams['font.family'] = 'Malgun Gothic'
except RuntimeError:
    print("Malgun Gothic 폰트를 찾을 수 없습니다. 기본 폰트를 사용합니다.")
plt.rcParams['axes.unicode_minus'] = False # 마이너스 기호 깨짐 방지

# 예제 데이터 로드 (Seaborn에 내장된 'tips' 데이터셋)
tips = sns.load_dataset("tips")

# --- Matplotlib & Seaborn을 활용한 다양한 시각화 ---

# 1. 막대 그래프 (Bar Plot): 카테고리별 데이터 비교
plt.figure(figsize=(10, 6))
sns.barplot(x="day", y="total_bill", hue="sex", data=tips, palette="viridis")
plt.title("요일 및 성별 전체 결제 금액 평균", fontsize=15)
plt.xlabel("요일")
plt.ylabel("전체 결제 금액 ($)")
plt.show()

# 2. 산점도 (Scatter Plot): 두 연속형 변수 간의 관계
# 점의 크기(size), 모양(style), 색상(hue)으로 추가적인 정보를 표현할 수 있습니다.
plt.figure(figsize=(10, 6))
sns.scatterplot(x="total_bill", y="tip", hue="time", style="smoker", size="size", data=tips, alpha=0.7, palette="muted")
plt.title("결제 금액과 팁의 관계 (시간/흡연여부/인원수별)", fontsize=15)
plt.xlabel("전체 결제 금액 ($)")
plt.ylabel("팁 ($)")
plt.legend(title="구분")
plt.show()

# 3. 히스토그램 (Histogram)과 KDE: 단일 변수의 분포 확인
plt.figure(figsize=(10, 6))
sns.histplot(tips['total_bill'], kde=True, bins=30, color='skyblue')
plt.title("전체 결제 금액의 분포", fontsize=15)
plt.xlabel("전체 결제 금액 ($)")
plt.ylabel("빈도수")
plt.show()

# 4. 박스 플롯 (Box Plot): 그룹별 데이터 분포 요약(중앙값, 사분위수 등)
plt.figure(figsize=(10, 6))
sns.boxplot(x="day", y="tip", hue="smoker", data=tips, palette="Set3")
plt.title("요일 및 흡연여부별 팁의 분포", fontsize=15)
plt.xlabel("요일")
plt.ylabel("팁 ($)")
plt.show()
```
---
## 4. Scikit-learn - 실용적인 머신러닝

- **핵심**: 분류(Classification), 회귀(Regression), 군집화(Clustering) 등 전통적인 머신러닝 알고리즘 대부분을 일관되고 사용하기 쉬운 API로 제공하는 최고의 라이브러리입니다. 모델 학습, 평가, 데이터 분할, 특징 공학(Feature Engineering) 등 머신러닝 프로젝트의 전 과정을 지원합니다.

### 예제: 붓꽃(Iris) 품종 분류
```python
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import accuracy_score
from sklearn.datasets import load_iris
import joblib

# 1. 데이터 준비
iris = load_iris()
X = iris.data   # 특징(Feature): 꽃받침 길이/너비, 꽃잎 길이/너비
y = iris.target # 타겟(Label): 붓꽃 품종 (0, 1, 2)

# 2. 데이터 분리 (학습용 70% / 테스트용 30%)
# random_state: 재현 가능하도록 난수 시드 고정
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.3, random_state=42)
print(f"학습 데이터 shape: {X_train.shape}, 테스트 데이터 shape: {X_test.shape}")

# 3. 모델 선택 및 학습
# 로지스틱 회귀 모델 객체 생성
model = LogisticRegression(max_iter=200)
# 학습 데이터를 이용하여 모델 학습
model.fit(X_train, y_train)

# 4. 예측 및 평가
# 학습된 모델로 테스트 데이터 예측
predictions = model.predict(X_test)
# 실제값(y_test)과 예측값(predictions)을 비교하여 정확도 계산
accuracy = accuracy_score(y_test, predictions)
print(f"모델 예측 정확도: {accuracy:.4f}")

# 5. 모델 저장하기 (Part 6에서 사용)
# 학습된 모델을 나중에 재사용하기 위해 파일로 저장합니다.
joblib.dump(model, 'iris_model.pkl')
print("학습된 모델을 'iris_model.pkl' 파일로 저장했습니다.")
```

### 심화 예제: 보스턴 집값 예측 (회귀)

분류(Classification)가 정해진 카테고리 중 하나를 맞추는 문제라면, 회귀(Regression)는 연속적인 숫자 값을 예측하는 문제입니다. Scikit-learn은 회귀 문제 역시 분류와 거의 동일한 프로세스로 해결할 수 있도록 일관된 API를 제공합니다.

> ⚠️ **데이터셋 로드 주의사항**: `load_boston` 함수는 Scikit-learn 1.2 버전부터 사용이 중단될 예정입니다. 여기서는 데이터의 윤리적 문제를 인지하고 학습 목적으로만 사용하며, `fetch_openml`을 통해 대체 데이터를 로드합니다.

```python
from sklearn.model_selection import train_test_split
from sklearn.linear_model import Lasso
from sklearn.metrics import mean_squared_error
from sklearn.datasets import fetch_openml
import numpy as np
import joblib

# 1. 데이터 준비 (보스턴 집값 데이터셋)
# 'return_X_y=True'는 특징(X)과 타겟(y)을 튜플로 바로 반환해줍니다.
X, y = fetch_openml(name='boston', version=1, as_frame=False, return_X_y=True, parser='auto')

# 2. 데이터 분리
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.3, random_state=42)

# 3. 모델 선택 및 학습 (Lasso 회귀 모델 사용)
# Lasso는 불필요한 특징의 가중치를 0으로 만들어 모델을 단순화하는 효과가 있습니다.
model_boston = Lasso(alpha=0.1)
model_boston.fit(X_train, y_train)

# 4. 예측 및 평가
# 회귀 모델의 평가지표로는 주로 평균 제곱 오차(Mean Squared Error)를 사용합니다.
predictions_boston = model_boston.predict(X_test)
mse = mean_squared_error(y_test, predictions_boston)
print(f"\n[회귀 모델] 예측 평균 제곱 오차(MSE): {mse:.2f}")

# 5. 모델 저장하기
joblib.dump(model_boston, 'boston_model.pkl')
print("학습된 회귀 모델을 'boston_model.pkl' 파일로 저장했습니다.")
```

---

## 5. TensorFlow & PyTorch - 딥러닝 프레임워크

- **핵심**: 인간의 뇌를 모방한 인공 신경망(Artificial Neural Network)을 구축하고 학습시키기 위한 핵심 프레임워크입니다. 자동 미분(Autograd), GPU 가속 등 딥러닝에 필수적인 기능을 제공하며, 현대 AI 연구와 서비스의 중심에 있습니다. `PyTorch`는 연구 커뮤니티에서 선호되며, 유연하고 파이썬 친화적인 코딩 스타일이 특징입니다. `TensorFlow`는 강력한 생태계와 프로덕션 환경에서의 배포(TensorFlow Serving, TF Lite)에 강점을 가집니다.

### 5.1. PyTorch 예제

#### 텐서 연산과 자동 미분
```python
import torch

# 텐서(Tensor) 생성. requires_grad=True는 자동 미분을 추적하도록 설정
tensor_a = torch.tensor([[1, 2], [3, 4]], dtype=torch.float32, requires_grad=True)
tensor_b = torch.tensor([[5, 6], [7, 8]], dtype=torch.float32)

# 텐서 연산
tensor_sum = tensor_a + tensor_b
tensor_mul = tensor_a * tensor_b
# 행렬 곱
tensor_matmul = torch.matmul(tensor_a, tensor_b)

print(f"PyTorch 텐서의 합:\n{tensor_sum}")

# 딥러닝 모델의 손실(loss) 계산 과정 예시
loss = tensor_mul.sum()
# 손실에 대한 tensor_a의 기울기 계산
loss.backward()

# 기울기 확인
print(f"tensor_a의 기울기(gradient):\n{tensor_a.grad}")
```

#### 간단한 신경망 모델 정의
```python
import torch
import torch.nn as nn
import torch.nn.functional as F

# nn.Module을 상속받아 신경망 모델 클래스를 정의합니다.
class SimpleNet(nn.Module):
    def __init__(self, input_size, hidden_size, num_classes):
        super(SimpleNet, self).__init__()
        self.fc1 = nn.Linear(input_size, hidden_size)  # 첫 번째 완전 연결 레이어
        self.relu = nn.ReLU()                          # 활성화 함수
        self.fc2 = nn.Linear(hidden_size, num_classes) # 두 번째 완전 연결 레이어

    def forward(self, x):
        # 입력 데이터(x)가 모델을 통과하는 순방향 전파 로직을 정의합니다.
        out = self.fc1(x)
        out = self.relu(out)
        out = self.fc2(out)
        # Softmax는 보통 손실 함수(CrossEntropyLoss)에 포함되어 있어 생략 가능
        return out

# 모델 인스턴스 생성 (예: 붓꽃 데이터의 경우)
# 입력 특징 4개, 은닉층 노드 10개, 출력 클래스 3개
model = SimpleNet(input_size=4, hidden_size=10, num_classes=3)

# 모델 구조 출력
print("\nPyTorch 모델 구조:")
print(model)

# 임의의 입력 데이터로 테스트
dummy_input = torch.randn(1, 4) # (배치 크기, 입력 크기)
output = model(dummy_input)
print(f"\n임의 입력에 대한 모델 출력 (before softmax):\n{output}")
```

### 5.2. TensorFlow 예제

#### 텐서 연산과 자동 미분
```python
import tensorflow as tf

# 텐서(Tensor) 생성. tf.Variable은 학습 과정에서 업데이트될 변수를 의미
tensor_a = tf.Variable([[1, 2], [3, 4]], dtype=tf.float32)
tensor_b = tf.constant([[5, 6], [7, 8]], dtype=tf.float32)

with tf.GradientTape() as tape:
    # 텐서 연산
    tensor_sum = tf.add(tensor_a, tensor_b)
    tensor_mul = tf.multiply(tensor_a, tensor_b)
    # 행렬 곱
    tensor_matmul = tf.matmul(tensor_a, tensor_b)
    
    # 딥러닝 모델의 손실(loss) 계산 과정 예시
    loss = tf.reduce_sum(tensor_mul)

# 손실에 대한 tensor_a의 기울기 계산
gradients = tape.gradient(loss, tensor_a)

print(f"TensorFlow 텐서의 합:\n{tensor_sum.numpy()}")
print(f"tensor_a의 기울기(gradient):\n{gradients.numpy()}")
```

#### 간단한 신경망 모델 정의
```python
import tensorflow as tf
from tensorflow.keras import Model
from tensorflow.keras.layers import Dense, Input

# Keras의 Model 클래스를 상속받아 신경망 모델을 정의합니다.
class SimpleNetTF(Model):
    def __init__(self, hidden_size, num_classes):
        super(SimpleNetTF, self).__init__()
        self.fc1 = Dense(hidden_size, activation='relu') # 완전 연결 레이어
        self.fc2 = Dense(num_classes)                    # 출력 레이어

    def call(self, x):
        # 입력 데이터(x)가 모델을 통과하는 순방향 전파 로직을 정의합니다.
        x = self.fc1(x)
        return self.fc2(x)

# 모델 인스턴스 생성 (예: 붓꽃 데이터의 경우)
model_tf = SimpleNetTF(hidden_size=10, num_classes=3)

# Keras는 모델에 첫 입력을 주기 전까지는 가중치(weight)를 생성하지 않습니다.
# 입력 데이터의 형태를 지정하여 모델을 빌드합니다.
model_tf.build(input_shape=(None, 4)) # (배치 크기, 입력 크기)

# 모델 구조 출력
print("\nTensorFlow/Keras 모델 구조:")
model_tf.summary()

# 임의의 입력 데이터로 테스트
dummy_input = tf.random.normal([1, 4])
output = model_tf(dummy_input)
print(f"\n임의 입력에 대한 모델 출력 (before softmax):\n{output.numpy()}")
```

### 5.3. TensorFlow 모델 정의 (PyTorch와 유사한 구조)
```python
import tensorflow as tf

# TensorFlow 모델 정의 (PyTorch와 유사한 구조)
model_tf = tf.keras.Sequential([
    tf.keras.layers.Dense(10, activation='relu', input_shape=(4,)),
    tf.keras.layers.Dense(10, activation='relu'),
    tf.keras.layers.Dense(3, activation='softmax')
])
# 모델 컴파일
model_tf.compile(optimizer='adam',
               loss='sparse_categorical_crossentropy',
               metrics=['accuracy'])
# 모델 학습
# model_tf.fit(X_train, y_train, epochs=50, batch_size=5, verbose=0)
```
> 이 파트에서는 TensorFlow와 PyTorch의 기본 개념만 간단히 소개했습니다. 딥러닝은 별도의 심화 과정이 필요할 만큼 방대한 주제이며, Part 8에서 더 자세한 학습 로드맵을 안내합니다.

---

## 6. 미니 프로젝트: 타이타닉 생존자 데이터 분석

지금까지 배운 NumPy, Pandas, Matplotlib, Seaborn 라이브러리를 총동원하여, 데이터 분석의 "Hello, World"와도 같은 캐글(Kaggle)의 타이타닉 데이터셋을 분석하는 미니 프로젝트를 진행해 보겠습니다.

> **데이터셋**: 이 프로젝트는 Seaborn에 내장된 'titanic' 데이터셋을 사용합니다. `sns.load_dataset('titanic')` 코드로 쉽게 불러올 수 있습니다.

### 6.1. 프로젝트 목표

타이타닉호 승객 데이터를 분석하여, **"어떤 승객이 생존할 확률이 높았을까?"** 라는 질문에 대한 답을 데이터 기반으로 찾아보고, 그 과정과 결과를 시각화하여 설득력 있게 제시하는 것을 목표로 합니다.

### 6.2. 분석 과제 (Missions)

아래의 질문들에 대한 답을 찾기 위해, Pandas로 데이터를 탐색하고 Matplotlib/Seaborn으로 결과를 시각화하는 코드를 직접 작성해 보세요.

1.  **전체 생존율**: 전체 승객 중 몇 퍼센트가 생존했을까요? (파이 차트)
2.  **성별에 따른 생존율**: 남성과 여성 중 어느 쪽의 생존율이 더 높았을까요? (막대 그래프)
3.  **객실 등급(Pclass)에 따른 생존율**: 1, 2, 3등석 객실 승객들의 생존율은 어떻게 달랐을까요? (막대 그래프)
4.  **나이에 따른 생존율**: 나이 분포와 생존율의 관계는 어떠할까요? (히스토그램, KDE 플롯)
5.  **종합 분석**: 위 분석들을 종합해볼 때, 어떤 특징을 가진 승객의 생존율이 가장 높았다고 결론 내릴 수 있을까요? (예: `성별`과 `객실 등급`을 함께 고려한 복합 분석)

### 6.3. 예시 코드 (Missions 1 & 2)

```python
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

# Seaborn 스타일 및 한글 폰트 설정
sns.set_theme(style="whitegrid")
try:
    plt.rcParams['font.family'] = 'Malgun Gothic'
except RuntimeError:
    print("Malgun Gothic 폰트를 찾을 수 없습니다. 기본 폰트를 사용합니다.")
plt.rcParams['axes.unicode_minus'] = False

# 1. 데이터 로드 및 기본 정보 확인
titanic_df = sns.load_dataset('titanic')

print("--- 데이터 샘플 (상위 5개) ---")
print(titanic_df.head())
print("\n--- 데이터 기본 정보 ---")
titanic_df.info()

# 2. 데이터 전처리 (결측치 채우기)
# 'age' 컬럼의 결측치를 전체 승객의 나이 평균으로 채웁니다.
mean_age = titanic_df['age'].mean()
titanic_df['age'].fillna(mean_age, inplace=True)
print(f"\n'age' 컬럼의 결측치를 평균값({mean_age:.1f})으로 채웠습니다.")

# --- Mission 1: 전체 생존율 (파이 차트) ---
survived_counts = titanic_df['survived'].value_counts()
labels = ['사망 (0)', '생존 (1)']

plt.figure(figsize=(8, 8))
plt.pie(survived_counts, labels=labels, autopct='%1.1f%%', startangle=90, colors=['#ff9999','#66b3ff'])
plt.title('타이타닉 전체 생존율', fontsize=16)
plt.ylabel('') # 불필요한 라벨 제거
plt.show()

# --- Mission 2: 성별에 따른 생존율 (막대 그래프) ---
plt.figure(figsize=(10, 6))
sns.barplot(x='sex', y='survived', data=titanic_df, palette='pastel')
plt.title('성별에 따른 생존율', fontsize=16)
plt.xlabel('성별 (sex)', fontsize=12)
plt.ylabel('생존율 (survived)', fontsize=12)
# y축을 퍼센트로 표시
plt.gca().yaxis.set_major_formatter(plt.FuncFormatter(lambda y, _: f'{y:.0%}'))
plt.show()

# --- Mission 3, 4, 5는 직접 도전해보세요! ---
# 힌트: Mission 2의 코드를 참고하여 x축을 'pclass', 'age' 등으로 변경하고,
# hue 옵션을 추가하여 복합적인 분석을 수행할 수 있습니다.
# sns.histplot(data=titanic_df, x='age', hue='survived', multiple='stack', bins=30)
# sns.catplot(x='pclass', y='survived', hue='sex', data=titanic_df, kind='bar')

```

이 프로젝트를 통해 여러분은 실제 데이터를 정제하고, 분석 질문을 설정하며, 적절한 시각화를 통해 인사이트를 도출하는 데이터 분석의 전 과정을 경험하게 될 것입니다.

---

### ✅ 정리 및 다음 단계

이번 파트에서는 AI 개발의 실질적인 도구인 핵심 라이브러리들을 익혔습니다.

- **NumPy**: 고성능 수치 계산을 위한 배열 처리
- **Pandas**: 테이블 형태의 데이터를 다루기 위한 데이터프레임
- **Matplotlib/Seaborn**: 데이터의 의미를 파악하기 위한 시각화
- **Scikit-learn**: 전통적인 머신러닝 모델의 학습 및 평가

특히, 우리는 Scikit-learn을 사용하여 **붓꽃 품종을 예측하는 모델을 직접 학습시키고, 그 결과를 `iris_model.pkl` 파일로 저장**하는 중요한 실습을 완료했습니다.

이제 우리는 데이터로부터 지식을 추출하고, 예측 모델을 만들어 파일로 저장할 수 있는 능력을 갖추게 되었습니다. 하지만 이 모델은 아직 개발자의 컴퓨터에만 존재합니다.

**➡️ 다음 시간: [Part 6: AI 모델 서비스화 기초 (FastAPI)](part_6_model_serving_with_fastapi.md)**

다음 시간에는 우리가 만든 `iris_model.pkl` 모델을 실제 세상과 연결하는 다리를 놓습니다. **FastAPI**라는 강력한 웹 프레임워크를 사용하여, 누구나 요청을 보내고 예측 결과를 받아볼 수 있는 **실시간 API 서버**를 구축하는 방법을 배울 것입니다. 